@@ -1,6 +1,6 @@
 /* pic32mz-crypt.c
  *
- * Copyright (C) 2006-2017 wolfSSL Inc.
+ * Copyright (C) 2006-2023 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
@@ -75,7 +75,7 @@ static int Pic32GetBlockSize(int algo)
     return 0;
 }
 
-static int Pic32Crypto(const byte* in, int inLen, word32* out, int outLen,
+static int Pic32Crypto(const byte* pIn, int inLen, word32* pOut, int outLen,
     int dir, int algo, int cryptoalgo,
 
     /* For DES/AES only */
@@ -89,9 +89,14 @@ static int Pic32Crypto(const byte* in, int inLen, word32* out, int outLen,
     bufferDescriptor *bd_p;
     byte *in_p;
     byte *out_p;
+    unsigned int *updptrVal_p;
     word32* dst;
     word32 padRemain;
     int timeout = 0xFFFFFF;
+    word32* in = (word32*)pIn;
+    word32* out = pOut;
+    word32 updptrVal = 0;
+    int isDynamic = 0;
 
     /* check args */
     if (in == NULL || inLen <= 0 || out == NULL || blockSize == 0) {
@@ -100,7 +105,21 @@ static int Pic32Crypto(const byte* in, int inLen, word32* out, int outLen,
 
     /* check pointer alignment - must be word aligned */
     if (((size_t)in % sizeof(word32)) || ((size_t)out % sizeof(word32))) {
-        return BUFFER_E; /* buffer is not aligned */
+        /* dynamically allocate aligned pointers */
+        isDynamic = 1;
+        in = (word32*)XMALLOC(inLen, NULL, DYNAMIC_TYPE_AES_BUFFER);
+        if (in == NULL)
+            return MEMORY_E;
+        if ((word32*)pIn == pOut) /* inline */
+            out = (word32*)in;
+        else {
+            out = (word32*)XMALLOC(outLen, NULL, DYNAMIC_TYPE_AES_BUFFER);
+            if (out == NULL) {
+                XFREE(in, NULL, DYNAMIC_TYPE_AES_BUFFER);
+                return MEMORY_E;
+            }
+        }
+        XMEMCPY(in, pIn, inLen);
     }
 
     /* get uncached address */
@@ -108,6 +127,7 @@ static int Pic32Crypto(const byte* in, int inLen, word32* out, int outLen,
     bd_p = KVA0_TO_KVA1(&bd);
     out_p= KVA0_TO_KVA1(out);
     in_p = KVA0_TO_KVA1(in);
+    updptrVal_p = KVA0_TO_KVA1(&updptrVal);
 
     /* Sync cache if in physical memory (not flash) */
     if (PIC32MZ_IF_RAM(in_p)) {
@@ -173,9 +193,10 @@ static int Pic32Crypto(const byte* in, int inLen, word32* out, int outLen,
     bd_p->SRCADDR = (unsigned int)KVA_TO_PA(in);
     if (key) {
         /* cipher */
-        if (in != (byte*)out)
+        if (in != out)
             XMEMSET(out_p, 0, outLen); /* clear output buffer */
         bd_p->DSTADDR = (unsigned int)KVA_TO_PA(out);
+        bd_p->UPDPTR = (unsigned int)KVA_TO_PA(updptrVal_p);
     }
     else {
         /* hashing */
@@ -224,20 +245,6 @@ static int Pic32Crypto(const byte* in, int inLen, word32* out, int outLen,
 
         wolfSSL_CryptHwMutexUnLock();
 
-        if (iv && ivLen > 0) {
-            /* set iv for the next call */
-            if (dir == PIC32_ENCRYPTION) {
-                XMEMCPY(iv, KVA0_TO_KVA1(out + (outLen - ivLen)), ivLen);
-            #if !PIC32_NO_OUT_SWAP
-                /* hardware already swapped output, so we need to swap back */
-                ByteReverseWords(iv, iv, ivLen);
-            #endif
-            }
-            else {
-                ByteReverseWords(iv, KVA0_TO_KVA1(in + (inLen - ivLen)), ivLen);
-            }
-        }
-
         /* copy result to output */
     #if PIC32_NO_OUT_SWAP
         /* swap bytes */
@@ -250,6 +257,17 @@ static int Pic32Crypto(const byte* in, int inLen, word32* out, int outLen,
     #endif
     }
 
+    /* handle unaligned */
+    if (isDynamic) {
+        /* return result */
+        XMEMCPY(pOut, out, outLen);
+
+        /* free dynamic buffers */
+        XFREE(in, NULL, DYNAMIC_TYPE_AES_BUFFER);
+        if ((word32*)pIn != pOut)
+            XFREE(out, NULL, DYNAMIC_TYPE_AES_BUFFER);
+    }
+
     return ret;
 }
 #endif /* WOLFSSL_PIC32MZ_CRYPT || WOLFSSL_PIC32MZ_HASH */
@@ -287,13 +305,12 @@ typedef struct {
     securityAssociation         sa                 __attribute__((aligned (8)));
 } pic32mz_desc;
 
-static pic32mz_desc gLHDesc;
+static pic32mz_desc gLHDesc __attribute__((coherent));
 static uint8_t gLHDataBuf[PIC32MZ_MAX_BD][PIC32_BLOCK_SIZE] __attribute__((aligned (4), coherent));
 
-static void reset_engine(pic32mz_desc *desc, int algo)
+static void reset_engine(int algo)
 {
     int i;
-    pic32mz_desc* uc_desc = KVA0_TO_KVA1(desc);
 
     wolfSSL_CryptHwMutexLock();
 
@@ -305,37 +322,36 @@ static void reset_engine(pic32mz_desc *desc, int algo)
     CEINTSRC = 0xF;
 
     /* Make sure everything is clear first before we setup */
-    XMEMSET(desc, 0, sizeof(pic32mz_desc));
-    XMEMSET((void *)&uc_desc->sa, 0, sizeof(uc_desc->sa));
+    XMEMSET(&gLHDesc, 0, sizeof(pic32mz_desc));
 
     /* Set up the Security Association */
-    uc_desc->sa.SA_CTRL.ALGO = algo;
-    uc_desc->sa.SA_CTRL.LNC = 1;
-    uc_desc->sa.SA_CTRL.FB = 1;
-    uc_desc->sa.SA_CTRL.ENCTYPE = 1;
-    uc_desc->sa.SA_CTRL.LOADIV = 1;
+    gLHDesc.sa.SA_CTRL.ALGO = algo;
+    gLHDesc.sa.SA_CTRL.LNC = 1;
+    gLHDesc.sa.SA_CTRL.FB = 1;
+    gLHDesc.sa.SA_CTRL.ENCTYPE = 1;
+    gLHDesc.sa.SA_CTRL.LOADIV = 1;
 
     /* Set up the Buffer Descriptor */
-    uc_desc->err = 0;
+    gLHDesc.err = 0;
     for (i = 0; i < PIC32MZ_MAX_BD; i++) {
-        XMEMSET((void *)&uc_desc->bd[i], 0, sizeof(uc_desc->bd[i]));
-        uc_desc->bd[i].BD_CTRL.LAST_BD = 1;
-        uc_desc->bd[i].BD_CTRL.LIFM = 1;
-        uc_desc->bd[i].BD_CTRL.PKT_INT_EN = 1;
-        uc_desc->bd[i].SA_ADDR = KVA_TO_PA(&uc_desc->sa);
-        uc_desc->bd[i].SRCADDR = KVA_TO_PA(&gLHDataBuf[i]);
+        XMEMSET((void *)&gLHDesc.bd[i], 0, sizeof(gLHDesc.bd[i]));
+        gLHDesc.bd[i].BD_CTRL.LAST_BD = 1;
+        gLHDesc.bd[i].BD_CTRL.LIFM = 1;
+        gLHDesc.bd[i].BD_CTRL.PKT_INT_EN = 1;
+        gLHDesc.bd[i].SA_ADDR = KVA_TO_PA(&gLHDesc.sa);
+        gLHDesc.bd[i].SRCADDR = KVA_TO_PA(&gLHDataBuf[i]);
         if (PIC32MZ_MAX_BD > i+1)
-            uc_desc->bd[i].NXTPTR = KVA_TO_PA(&uc_desc->bd[i+1]);
+            gLHDesc.bd[i].NXTPTR = KVA_TO_PA(&gLHDesc.bd[i+1]);
         else
-            uc_desc->bd[i].NXTPTR = KVA_TO_PA(&uc_desc->bd[0]);
+            gLHDesc.bd[i].NXTPTR = KVA_TO_PA(&gLHDesc.bd[0]);
         XMEMSET((void *)&gLHDataBuf[i], 0, PIC32_BLOCK_SIZE);
     }
-    uc_desc->bd[0].BD_CTRL.SA_FETCH_EN = 1; /* Fetch the security association on the first BD */
-    desc->dbPtr = 0;
-    desc->currBd = 0;
-    desc->msgSize = 0;
-    desc->processed = 0;
-    CEBDPADDR = KVA_TO_PA(&(desc->bd[0]));
+    gLHDesc.bd[0].BD_CTRL.SA_FETCH_EN = 1; /* Fetch the security association on the first BD */
+    gLHDesc.dbPtr = 0;
+    gLHDesc.currBd = 0;
+    gLHDesc.msgSize = 0;
+    gLHDesc.processed = 0;
+    CEBDPADDR = KVA_TO_PA(&(gLHDesc.bd[0]));
 
     CEPOLLCON = 10;
 
@@ -346,13 +362,11 @@ static void reset_engine(pic32mz_desc *desc, int algo)
 #endif
 }
 
-static void update_engine(pic32mz_desc *desc, const byte *input, word32 len,
-    word32 *hash)
+static void update_engine(const byte *input, word32 len, word32 *hash)
 {
     int total;
-    pic32mz_desc *uc_desc = KVA0_TO_KVA1(desc);
-
-    uc_desc->bd[desc->currBd].UPDPTR = KVA_TO_PA(hash);
+    
+    gLHDesc.bd[gLHDesc.currBd].UPDPTR = KVA_TO_PA(hash);
 
     /* Add the data to the current buffer. If the buffer fills, start processing it
        and fill the next one. */
@@ -360,75 +374,76 @@ static void update_engine(pic32mz_desc *desc, const byte *input, word32 len,
         /* If we've been given the message size, we can process along the
            way.
            Enable the current buffer descriptor if it is full. */
-        if (desc->dbPtr >= PIC32_BLOCK_SIZE) {
+        if (gLHDesc.dbPtr >= PIC32_BLOCK_SIZE) {
             /* Wrap up the buffer descriptor and enable it so the engine can process */
-            uc_desc->bd[desc->currBd].MSGLEN = desc->msgSize;
-            uc_desc->bd[desc->currBd].BD_CTRL.BUFLEN = desc->dbPtr;
-            uc_desc->bd[desc->currBd].BD_CTRL.LAST_BD = 0;
-            uc_desc->bd[desc->currBd].BD_CTRL.LIFM = 0;
-            uc_desc->bd[desc->currBd].BD_CTRL.DESC_EN = 1;
+            gLHDesc.bd[gLHDesc.currBd].MSGLEN = gLHDesc.msgSize;
+            gLHDesc.bd[gLHDesc.currBd].BD_CTRL.BUFLEN = gLHDesc.dbPtr;
+            gLHDesc.bd[gLHDesc.currBd].BD_CTRL.LAST_BD = 0;
+            gLHDesc.bd[gLHDesc.currBd].BD_CTRL.LIFM = 0;
+            gLHDesc.bd[gLHDesc.currBd].BD_CTRL.DESC_EN = 1;
             /* Move to the next buffer descriptor, or wrap around. */
-            desc->currBd++;
-            if (desc->currBd >= PIC32MZ_MAX_BD)
-                desc->currBd = 0;
+            gLHDesc.currBd++;
+            if (gLHDesc.currBd >= PIC32MZ_MAX_BD)
+                gLHDesc.currBd = 0;
             /* Wait until the engine has processed the new BD. */
-            while (uc_desc->bd[desc->currBd].BD_CTRL.DESC_EN);
-            uc_desc->bd[desc->currBd].UPDPTR = KVA_TO_PA(hash);
-            desc->dbPtr = 0;
+            while (gLHDesc.bd[gLHDesc.currBd].BD_CTRL.DESC_EN);
+            gLHDesc.bd[gLHDesc.currBd].UPDPTR = KVA_TO_PA(hash);
+            gLHDesc.dbPtr = 0;
         }
         if (!PIC32MZ_IF_RAM(input)) {
             /* If we're inputting from flash, let the BD have
                the address and max the buffer size */
-            uc_desc->bd[desc->currBd].SRCADDR = KVA_TO_PA(input);
+            gLHDesc.bd[gLHDesc.currBd].SRCADDR = KVA_TO_PA(input);
             total = (len > PIC32MZ_MAX_BLOCK ? PIC32MZ_MAX_BLOCK : len);
-            desc->dbPtr = total;
+            gLHDesc.dbPtr = total;
             len -= total;
             input += total;
         }
         else {
-            if (len > PIC32_BLOCK_SIZE - desc->dbPtr) {
+            if (len > PIC32_BLOCK_SIZE - gLHDesc.dbPtr) {
                 /* We have more data than can be put in the buffer. Fill what we can.*/
-                total = PIC32_BLOCK_SIZE - desc->dbPtr;
-                XMEMCPY(&gLHDataBuf[desc->currBd][desc->dbPtr], input, total);
+                total = PIC32_BLOCK_SIZE - gLHDesc.dbPtr;
+                XMEMCPY(&gLHDataBuf[gLHDesc.currBd][gLHDesc.dbPtr], input, total);
                 len -= total;
-                desc->dbPtr = PIC32_BLOCK_SIZE;
+                gLHDesc.dbPtr = PIC32_BLOCK_SIZE;
                 input += total;
             }
             else {
                 /* Fill up what we have, but don't turn on the engine.*/
-                XMEMCPY(&gLHDataBuf[desc->currBd][desc->dbPtr], input, len);
-                desc->dbPtr += len;
+                XMEMCPY(&gLHDataBuf[gLHDesc.currBd][gLHDesc.dbPtr], input, len);
+                gLHDesc.dbPtr += len;
                 len = 0;
             }
         }
     }
 }
 
-static void start_engine(pic32mz_desc *desc)
+static void start_engine(void)
 {
     /* Wrap up the last buffer descriptor and enable it */
     int bufferLen;
-    pic32mz_desc *uc_desc = KVA0_TO_KVA1(desc);
 
-    bufferLen = desc->dbPtr;
+    bufferLen = gLHDesc.dbPtr;
     if (bufferLen % 4)
         bufferLen = (bufferLen + 4) - (bufferLen % 4);
-    uc_desc->bd[desc->currBd].BD_CTRL.BUFLEN = bufferLen;
-    uc_desc->bd[desc->currBd].BD_CTRL.LAST_BD = 1;
-    uc_desc->bd[desc->currBd].BD_CTRL.LIFM = 1;
-    uc_desc->bd[desc->currBd].BD_CTRL.DESC_EN = 1;
+    /* initialize the MSGLEN on engine startup to avoid infinite loop when
+     * length is less than 257 (size of PIC32_BLOCK_SIZE) */
+    gLHDesc.bd[gLHDesc.currBd].MSGLEN = gLHDesc.msgSize;
+    gLHDesc.bd[gLHDesc.currBd].BD_CTRL.BUFLEN = bufferLen;
+    gLHDesc.bd[gLHDesc.currBd].BD_CTRL.LAST_BD = 1;
+    gLHDesc.bd[gLHDesc.currBd].BD_CTRL.LIFM = 1;
+    gLHDesc.bd[gLHDesc.currBd].BD_CTRL.DESC_EN = 1;
 }
 
-void wait_engine(pic32mz_desc *desc, char *hash, int hash_sz)
+void wait_engine(char *hash, int hash_sz)
 {
     int i;
-    pic32mz_desc *uc_desc = KVA0_TO_KVA1(desc);
     unsigned int engineRunning;
 
     do {
         engineRunning = 0;
         for (i = 0; i < PIC32MZ_MAX_BD; i++) {
-            engineRunning = engineRunning || uc_desc->bd[i].BD_CTRL.DESC_EN;
+            engineRunning = engineRunning || gLHDesc.bd[i].BD_CTRL.DESC_EN;
         }
     } while (engineRunning);
 
@@ -472,10 +487,10 @@ static int wc_Pic32HashUpdate(hashUpdCache* cache, byte* stdBuf, int stdBufLen,
     /* if final length is set then pass straight to hardware */
     if (cache->finalLen) {
         if (cache->bufLen == 0) {
-            reset_engine(&gLHDesc, algo);
+            reset_engine(algo);
             gLHDesc.msgSize = cache->finalLen;
         }
-        update_engine(&gLHDesc, data, len, digest);
+        update_engine(data, len, digest);
         cache->bufLen += len; /* track progress for blockType */
         return 0;
     }
@@ -519,6 +534,7 @@ static int wc_Pic32HashUpdate(hashUpdCache* cache, byte* stdBuf, int stdBufLen,
         XMEMCPY(newBuf, cache->buf, cache->updLen);
         if (isNewBuf && cache->buf != stdBuf) {
             XFREE(cache->buf, heap, DYNAMIC_TYPE_HASH_TMP);
+            cache->buf = NULL;
         }
     }
     XMEMCPY(newBuf + cache->updLen, data, len);
@@ -542,9 +558,16 @@ static int wc_Pic32HashFinal(hashUpdCache* cache, byte* stdBuf,
 
 #ifdef WOLFSSL_PIC32MZ_LARGE_HASH
     if (cache->finalLen) {
-        start_engine(&gLHDesc);
-        wait_engine(&gLHDesc, (char*)digest, digestSz);
-        XMEMCPY(hash, digest, digestSz);
+        /* Only submit to hardware if update data provided matches expected */
+        if (cache->bufLen == cache->finalLen) {
+            start_engine();
+            wait_engine((char*)digest, digestSz);
+            XMEMCPY(hash, digest, digestSz);
+        }
+        else {
+            wolfSSL_CryptHwMutexUnLock();
+            ret = BUFFER_E;
+        }
         cache->finalLen = 0;
     }
     else
@@ -586,6 +609,7 @@ static int wc_Pic32HashFinal(hashUpdCache* cache, byte* stdBuf,
 
         if (cache->buf && cache->buf != stdBuf && !cache->isCopy) {
             XFREE(cache->buf, heap, DYNAMIC_TYPE_HASH_TMP);
+            cache->buf = NULL;
         }
     }
 
@@ -595,20 +619,27 @@ static int wc_Pic32HashFinal(hashUpdCache* cache, byte* stdBuf,
     return ret;
 }
 
-/* API's for compatability with Harmony wrappers - not used */
+static void wc_Pic32HashFree(hashUpdCache* cache, void* heap)
+{
+    if (cache && cache->buf && !cache->isCopy) {
+        XFREE(cache->buf, heap, DYNAMIC_TYPE_HASH_TMP);
+        cache->buf = NULL;
+    }
+}
+
+/* API's for compatibility with Harmony wrappers - not used */
 #ifndef NO_MD5
-    int wc_InitMd5_ex(Md5* md5, void* heap, int devId)
+    int wc_InitMd5_ex(wc_Md5* md5, void* heap, int devId)
     {
         if (md5 == NULL)
             return BAD_FUNC_ARG;
 
-        XMEMSET(md5, 0, sizeof(Md5));
+        XMEMSET(md5, 0, sizeof(wc_Md5));
         md5->heap = heap;
         (void)devId;
         return 0;
     }
-
-    int wc_Md5Update(Md5* md5, const byte* data, word32 len)
+    int wc_Md5Update(wc_Md5* md5, const byte* data, word32 len)
     {
         if (md5 == NULL || (data == NULL && len > 0))
             return BAD_FUNC_ARG;
@@ -616,8 +647,7 @@ static int wc_Pic32HashFinal(hashUpdCache* cache, byte* stdBuf,
             sizeof(md5->buffer), md5->digest, MD5_DIGEST_SIZE,
             data, len, PIC32_ALGO_MD5, md5->heap);
     }
-
-    int wc_Md5Final(Md5* md5, byte* hash)
+    int wc_Md5Final(wc_Md5* md5, byte* hash)
     {
         int ret;
 
@@ -632,8 +662,7 @@ static int wc_Pic32HashFinal(hashUpdCache* cache, byte* stdBuf,
 
         return ret;
     }
-
-    void wc_Md5SizeSet(Md5* md5, word32 len)
+    void wc_Md5SizeSet(wc_Md5* md5, word32 len)
     {
         if (md5) {
         #ifdef WOLFSSL_PIC32MZ_LARGE_HASH
@@ -643,20 +672,25 @@ static int wc_Pic32HashFinal(hashUpdCache* cache, byte* stdBuf,
         #endif
         }
     }
+    void wc_Md5Pic32Free(wc_Md5* md5)
+    {
+        if (md5) {
+            wc_Pic32HashFree(&md5->cache, md5->heap);
+        }
+    }
 #endif /* !NO_MD5 */
 #ifndef NO_SHA
-    int wc_InitSha_ex(Sha* sha, void* heap, int devId)
+    int wc_InitSha_ex(wc_Sha* sha, void* heap, int devId)
     {
         if (sha == NULL)
             return BAD_FUNC_ARG;
 
-        XMEMSET(sha, 0, sizeof(Sha));
+        XMEMSET(sha, 0, sizeof(wc_Sha));
         sha->heap = heap;
         (void)devId;
         return 0;
     }
-
-    int wc_ShaUpdate(Sha* sha, const byte* data, word32 len)
+    int wc_ShaUpdate(wc_Sha* sha, const byte* data, word32 len)
     {
         if (sha == NULL || (data == NULL && len > 0))
             return BAD_FUNC_ARG;
@@ -664,8 +698,7 @@ static int wc_Pic32HashFinal(hashUpdCache* cache, byte* stdBuf,
             sizeof(sha->buffer), sha->digest, SHA_DIGEST_SIZE,
             data, len, PIC32_ALGO_SHA1, sha->heap);
     }
-
-    int wc_ShaFinal(Sha* sha, byte* hash)
+    int wc_ShaFinal(wc_Sha* sha, byte* hash)
     {
         int ret;
 
@@ -680,7 +713,7 @@ static int wc_Pic32HashFinal(hashUpdCache* cache, byte* stdBuf,
 
         return ret;
     }
-    void wc_ShaSizeSet(Sha* sha, word32 len)
+    void wc_ShaSizeSet(wc_Sha* sha, word32 len)
     {
         if (sha) {
         #ifdef WOLFSSL_PIC32MZ_LARGE_HASH
@@ -690,20 +723,25 @@ static int wc_Pic32HashFinal(hashUpdCache* cache, byte* stdBuf,
         #endif
         }
     }
+    void wc_ShaPic32Free(wc_Sha* sha)
+    {
+        if (sha) {
+            wc_Pic32HashFree(&sha->cache, sha->heap);
+        }
+    }
 #endif /* !NO_SHA */
 #ifndef NO_SHA256
-    int wc_InitSha256_ex(Sha256* sha256, void* heap, int devId)
+    int wc_InitSha256_ex(wc_Sha256* sha256, void* heap, int devId)
     {
         if (sha256 == NULL)
             return BAD_FUNC_ARG;
 
-        XMEMSET(sha256, 0, sizeof(Sha256));
+        XMEMSET(sha256, 0, sizeof(wc_Sha256));
         sha256->heap = heap;
         (void)devId;
         return 0;
     }
-
-    int wc_Sha256Update(Sha256* sha256, const byte* data, word32 len)
+    int wc_Sha256Update(wc_Sha256* sha256, const byte* data, word32 len)
     {
         if (sha256 == NULL || (data == NULL && len > 0))
             return BAD_FUNC_ARG;
@@ -711,8 +749,7 @@ static int wc_Pic32HashFinal(hashUpdCache* cache, byte* stdBuf,
             sizeof(sha256->buffer), sha256->digest, SHA256_DIGEST_SIZE,
             data, len, PIC32_ALGO_SHA256, sha256->heap);
     }
-
-    int wc_Sha256Final(Sha256* sha256, byte* hash)
+    int wc_Sha256Final(wc_Sha256* sha256, byte* hash)
     {
         int ret;
 
@@ -727,8 +764,7 @@ static int wc_Pic32HashFinal(hashUpdCache* cache, byte* stdBuf,
 
         return ret;
     }
-
-    void wc_Sha256SizeSet(Sha256* sha256, word32 len)
+    void wc_Sha256SizeSet(wc_Sha256* sha256, word32 len)
     {
         if (sha256) {
         #ifdef WOLFSSL_PIC32MZ_LARGE_HASH
@@ -738,8 +774,14 @@ static int wc_Pic32HashFinal(hashUpdCache* cache, byte* stdBuf,
         #endif
         }
     }
+    void wc_Sha256Pic32Free(wc_Sha256* sha256)
+    {
+        if (sha256) {
+            wc_Pic32HashFree(&sha256->cache, sha256->heap);
+        }
+    }
 #endif /* !NO_SHA256 */
-#endif
+#endif /* WOLFSSL_PIC32MZ_HASH */
 
 
 #ifdef WOLFSSL_PIC32MZ_CRYPT
